package Anglais;

import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.util.ArrayList;

import javax.swing.ImageIcon;
import javax.swing.JOptionPane;
import javax.swing.JTable;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.AbstractTableModel;

public class Editor extends AbstractTableModel {
	private static final long serialVersionUID = -7685340033452104169L;
	public static final int WORD1_INDEX = 0;
	public static final int WORD2_INDEX = 1;
	public static final int DEF_INDEX = 2;

	protected String[] columnNames;
	protected ArrayList<String[]> dataVector;
	protected JTable table;

	public Editor(String[] columnNames, JTable table) {
		this.columnNames = columnNames;
		this.table = table;
		final JTable ftable = table;
		
		dataVector = new ArrayList<String[]>();
		addTableModelListener(new InteractiveTableModelListener());

		table.addKeyListener(new KeyListener() {
			public void keyTyped(KeyEvent evt) {
				if(evt.getKeyChar() == KeyEvent.VK_DELETE) {
					if(JOptionPane.showConfirmDialog(null,
							"Delete these cards.\nAre you sure?", "Delete",
							JOptionPane.OK_CANCEL_OPTION, JOptionPane.QUESTION_MESSAGE, new ImageIcon("icon_omg.png")) == JOptionPane.OK_OPTION) {

						int row = ftable.getSelectedRow();
						ftable.setEditingColumn(2);
						
						dataVector.remove(row);
						fireTableRowsDeleted(row, row);
					}
				}
			}
			
			public void keyReleased(KeyEvent arg0) {}
			public void keyPressed(KeyEvent arg0) {}
		});
	}

	public ArrayList<String[]> getData() {
		return dataVector;
	}
	
	public void setData(ArrayList<String[]> data) {
		dataVector = data;
	}
	
	public String getColumnName(int column) {
		return columnNames[column];
	}

	public boolean isCellEditable(int row, int column) {
		return true;
	}

	public Object getValueAt(int row, int column) {
		String[] word = (String[]) dataVector.get(row);

		return word[column];
	}

	public void setValueAt(Object value, int row, int column) {
		String[] word = (String[]) dataVector.get(row);

		word[column] = ((String) value).replaceAll(",", "");
		fireTableCellUpdated(row, column);
		
		if((getRowCount() - 1) == row && !hasEmptyRow() &&
				!word[0].isEmpty() && !word[1].isEmpty() && !word[2].isEmpty()) {

			addEmptyRow();
			highlightLastRow(row);
		}
	}

	public int getRowCount() {
		return dataVector.size();
	}

	public int getColumnCount() {
		return columnNames.length;
	}

	public boolean hasEmptyRow() {
		if(dataVector.size() == 0)
			return false;

		String[] word = (String[]) dataVector.get(dataVector.size() - 1);
		
		return word[0].isEmpty() && word[1].isEmpty() && word[2].isEmpty(); 
	}

	public void addEmptyRow() {
		dataVector.add(new String[]{ "", "", "" });
;
		fireTableRowsInserted(
			dataVector.size() - 1,
			dataVector.size() - 1
		);
	}
	
	public void highlightLastRow(int row) {
		int lastrow = getRowCount();
		
		if(row == lastrow - 1)
			table.setRowSelectionInterval(lastrow - 1, lastrow - 1);
		else
			table.setRowSelectionInterval(row + 1, row + 1);

		table.setColumnSelectionInterval(0, 0);
	}

	public class InteractiveTableModelListener implements TableModelListener {
		public void tableChanged(TableModelEvent evt) {
			if(evt.getType() == TableModelEvent.UPDATE) {
				int column = evt.getColumn();
				int row = evt.getFirstRow();
				
				if(column != getColumnCount() - 1) {
					table.setColumnSelectionInterval(column + 1, column + 1);
					table.setRowSelectionInterval(row, row);
					
				} else if(row != getRowCount() - 1) {
					table.setColumnSelectionInterval(0, 0);
					table.setRowSelectionInterval(row + 1, row + 1);					
				}
			}
		}
	}
	
	public void loadDataFromFile(File f) {
		try {
			InputStream ips = new FileInputStream(f); 
			InputStreamReader ipsr = new InputStreamReader(ips);
			BufferedReader br = new BufferedReader(ipsr);
			String ligne;

			dataVector.clear();

			while((ligne = br.readLine()) != null) {
				String[] elmts = ligne.split("\\s*,\\s*", 3);

				if(elmts.length == 3)
					dataVector.add(elmts);
			}
			
			br.close();
		} catch(FileNotFoundException e) {
		} catch (IOException e) {
			JOptionPane.showMessageDialog(null, "File not readable!\nPlease, check permissions.", "Read error", JOptionPane.WARNING_MESSAGE, new ImageIcon("icon_def.png"));
		}
	}
		
	public void exportData(File f) {
		try {
			FileWriter fw = new FileWriter(f);
			BufferedWriter bw = new BufferedWriter(fw);
			PrintWriter fichierSortie = new PrintWriter(bw);
			
			ArrayList<String> words = new ArrayList<String>();
			
			for(String[] word : dataVector) {
				if(!word[0].isEmpty() && !word[1].isEmpty() && !word[2].isEmpty()
						&& !words.contains(word[0]) && !words.contains(word[1])) {

					fichierSortie.println(
						word[0] + ","
						+ word[1] + ","
						+ word[2]
					);

					words.add(word[0]);
					words.add(word[1]);
				}
			}

			fichierSortie.close();

		} catch (IOException e) {
			JOptionPane.showMessageDialog(null, "File not writable!\nPlease, check permissions.", "Write error", JOptionPane.WARNING_MESSAGE, new ImageIcon("icon_def.png"));
		}
	}
	
	// DOC: http://www.javalobby.org/articles/jtable/
}