package Anglais;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Observable;
import javax.swing.ImageIcon;
import javax.swing.JOptionPane;
import Anglais.VueMain.Update_t;

public class Plateau extends Observable {
	private int score;
	private ArrayList<Carte> base;
	private ArrayList<Carte> cartes;
	private ArrayList<String> defs;
	private Carte carte1, carte2;
	private Play play;
	private int size;
	private Game_t game_t;
	private int timer;
	private int timer_id;
	private int level;
	private String name;
	private boolean setLevel;
	
	private final String FICHIER_SYNONYMS = "db_synonyms";
	private final String FICHIER_TRANSLATE = "db_translate";
	private final String FICHIER_COLORS = "db_colors";
	private final String FICHIER_NUMBERS = "db_numbers";
	private final String FICHIER_PICTURES = "db_pictures";

	public enum Game_t { SYNONYMES, NUMBERS, COLORS, TRANSLATE, PICTURES };
	public final int TIME_MAX = 30;
	public final int SCORE_GOOD = +1;
	public final int SCORE_WRONG = -1;
	public final int SCORE_NEXT_LEVEL = +5;
	public final int SCORE_DEF = -2;

	public Play getPlay() {
		return play;
	}
	
	public Plateau() {
		play = new Play();		
		cartes = new ArrayList<Carte>();
		timer_id = 0;
		
		Thread thread = new Thread(play);
		thread.start();
	}
	
	public void start() {
		score = 0;

		if(loadCartes())
			nextLevel();
	}

	public void nextLevel() {
		level++;
		size = (int) Math.pow(2, level+1);
		
		if(!create())
			return;

		melanger();
		
		carte1 = null;
		carte2 = null;
		play.setCarte(null);
		
		setChanged();
		notifyObservers(Update_t.CREATE);
		
		score += getScoreNextLevel();
		
		setChanged();
		notifyObservers(Update_t.SCORE);
		
		new Thread(new Timer(++timer_id)).start();		
	}
	
	public int getScoreNextLevel() {
		return SCORE_NEXT_LEVEL*level;
	}
	
	public boolean create() {
		cartes = new ArrayList<Carte>();
		int j;
		
		if(size > base.size()) {
			JOptionPane.showMessageDialog(null, "Sorry, I can't!\nThere are not enough data in the db for this level.", "Clue", JOptionPane.INFORMATION_MESSAGE, new ImageIcon("icon_def.png"));
			return false;
		}
		
		for(int i = 0; i < size / 2; i++) {
			do {
				j = ((int) (Math.random() * 100 % base.size()) / 2) * 2;
			} while(cartes.contains(base.get(j)));

			base.get(j).setEtat(false);
			base.get(j+1).setEtat(false);
			
			cartes.add(base.get(j));
			cartes.add(base.get(j+1));
		}
		
		return true;
	}

	public String getFile() {
		return getFile(game_t);
	}
	
	public String getFile(Game_t game_t) {
		switch(game_t) {
			case COLORS:
				return FICHIER_COLORS;

			case TRANSLATE:
				return FICHIER_TRANSLATE;
				
			case NUMBERS:
				return FICHIER_NUMBERS;
				
			case PICTURES:
				return FICHIER_PICTURES;
	
			default:
				return FICHIER_SYNONYMS;
		}
	}
	
	public boolean loadCartes() {
		try{
			InputStream ips = new FileInputStream(getFile()); 
			InputStreamReader ipsr = new InputStreamReader(ips);
			BufferedReader br = new BufferedReader(ipsr);
			String ligne;

			int i = 0;
			
			base = new ArrayList<Carte>();
			defs = new ArrayList<String>();

			while((ligne = br.readLine()) != null){
				String[] elmts = ligne.split("\\s*,\\s*", 3);

				if(elmts.length == 3) {
					for(int j = 0; j < 2; j++) {
						if(!elmts[j].endsWith(".png"))
							elmts[j] = "<html><center>" + elmts[j].replace(" ", "<br>") + "</center></html>";
						
						base.add(new Carte(elmts[j], i));
					}

					defs.add(elmts[2]);

					i++;
				}
			}

			br.close();

		} catch (FileNotFoundException e) {
			JOptionPane.showMessageDialog(null, "No database found for this game.\nUse the editor to create one!", "Database not found", JOptionPane.WARNING_MESSAGE, new ImageIcon("icon_def.png"));
			return false;
			
		} catch (IOException e) {
			JOptionPane.showMessageDialog(null, "Database file not readable!\nPlease, check permissions.", "Database read error", JOptionPane.WARNING_MESSAGE, new ImageIcon("icon_def.png"));
			return false;
		}
		
		return true;
	}
	
	public boolean isSetLevel() {
		return setLevel;
	}

	public void setSetLevel(boolean setLevel) {
		this.setLevel = setLevel;
	}

	public int getSize() {
		return size;
	}
	
	public int getGame_t() {
		return size;
	}

	public void setGame_t(Game_t game_t) {
		this.game_t = game_t;
	}
	
	public int getTimer() {
		return timer;
	}

	public void setTimer(int timer) {
		this.timer = timer;
	}
	
	public String getDefinition(Carte c) {
		return defs.get(c.getValeur());
	}
	
	public boolean isCarte1(Carte c) {
		return carte1 == c;
	}

	public String getName() {
		return name;
	}
	
	public void setName(String name) {
		this.name = name;
		
		setChanged();
		notifyObservers(Update_t.SCORE);
	}
	
	public int getScore() {
		return score;
	}
	
	public void setScore(int score) {
		this.score = score;

		setChanged();
		notifyObservers(Update_t.SCORE);
	}
	
	public int getLevel() {
		return level;
	}
	
	public void setLevel(int level) {
		this.level = level;
	}

	void melanger() {
		for(int i = 0; i < cartes.size(); i++) {
			int j = (int) (Math.random() * 100 % cartes.size());
			Carte tmp = cartes.get(i);
			cartes.set(i, cartes.get(j));
			cartes.set(j, tmp);
		}
	}
	
	private void wrongPair() {
		score += SCORE_WRONG;
		
		setChanged();
		notifyObservers(Update_t.SCORE);

		carte1.setEtat(false);
		carte2.setEtat(false);
	}

	private void goodPair() {
		score += SCORE_GOOD;
		
		setChanged();
		notifyObservers(Update_t.SCORE);
	}

	ArrayList<Carte> getCartes() {
		return cartes;
	}
	
	Carte getCarte(int p) {
		return cartes.get(p);
	}
	
	boolean partieTerminee() {
		for(Carte c : cartes) {
			if(!c.getEtat())
				return false;
		}
		
		return true;
	}
	
	public int getTimeMax() {
		return TIME_MAX*level;
	}
	
	boolean tempsEcoule() {
		return timer >= getTimeMax();
	}
	
	int getNombreCarte(int p) {
		return cartes.size();
	}
	
	public void resetScore() {
		setChanged();
		notifyObservers(Update_t.SCORE);
	}
	
	public String[] getMeilleurScore(Game_t game_t) {
		String score = "";
		String level = "";
		String name = "";
		
		try{
			InputStream ips = new FileInputStream(getFile(game_t) + "_best"); 
			InputStreamReader ipsr = new InputStreamReader(ips);
			BufferedReader br = new BufferedReader(ipsr);

			name = br.readLine();
			score = br.readLine();
			level = br.readLine();

			br.close();

		} catch (Exception e) {
			e.printStackTrace();
		}
		
		return new String[] { name, score, level };
	}

	public boolean saveMeilleurScore() {
		String[] meilleur = getMeilleurScore(game_t);
		
		if(Integer.parseInt(meilleur[1]) > score)
			return false;
		
		saveMeilleurScore(game_t, name, score, level);
		
		return true;
	}
	
	public void saveMeilleurScore(Game_t game_t, String name, int score, int level) {
		try {
			FileWriter fw = new FileWriter(getFile(game_t) + "_best");
			BufferedWriter bw = new BufferedWriter(fw);
			PrintWriter fichierSortie = new PrintWriter(bw);
			
			fichierSortie.println(name);
			fichierSortie.println(score);
			fichierSortie.println(level);
			
			fichierSortie.close();
			
		} catch(Exception e) {
			e.printStackTrace();
		}
	}
	
	public class Play implements Runnable {
		Carte carte;
		
		public void setCarte(Carte carte) {
			this.carte = carte;
		}
		
		public boolean isRunning() {
			return carte != null;
		}
		
		@Override
		public void run() {
			while(true) {
				synchronized (this) {
					try {
						this.wait();
					} catch (InterruptedException e) {
						e.printStackTrace();
					}
				}
	
				carte.setEtat(true);
				
				if(partieTerminee()) {
					goodPair();
					
					carte1 = null;
					carte2 = null;
				}
				
				setChanged();
				notifyObservers(Update_t.SELECT);
				
				if(!partieTerminee()) {
					if(carte1 != null) {
						carte2 = carte;
						
						if(carte1.equals(carte2))
							goodPair();
						else {
							try {
								Thread.sleep(900);
							} catch (InterruptedException e) {
								e.printStackTrace();
							}
			
							wrongPair();
						}
						
						carte1 = null;
						carte2 = null;
			
						setChanged();
						notifyObservers(Update_t.SELECT);
	
					} else
						carte1 = carte;
				}
				
				carte = null;
			}
		}
	}
	
	public class Timer implements Runnable {
		private int id;
		
		public Timer(int id) {
			this.id = id;
		}
		
		@Override
		public void run() {
			timer = 0;
			
			try {
				while(!tempsEcoule() && !partieTerminee() && timer_id == id) {
					Thread.sleep(1000);
					timer++;
					
					setChanged();
					notifyObservers(Update_t.TIMER);
				}
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
		}
	}
}