package Anglais;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Container;
import java.util.Observable;
import java.util.Observer;
import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.UIManager;

public class VueMain extends JFrame implements Observer {
	private static final long serialVersionUID = 3988912754044528927L;
	private Plateau modele;
	private JProgressBar timer;
	private JLabel score;
	private VuePlateau plateau;
	
	public enum Update_t { CREATE, SELECT, TIMER, SCORE };
	
	public VueMain(Plateau modele) {
		this.modele = modele;		
		modele.addObserver(this);

		UIManager.put("MenuBar.foreground", new Color(235, 240, 208));
		UIManager.put("MenuBar.background", new Color(159, 0, 0));
		UIManager.put("Menu.foreground", new Color(235, 240, 208));
		UIManager.put("Menu.background", new Color(159, 0, 0));
		UIManager.put("Menu.selectionForeground", Color.WHITE);
		UIManager.put("Menu.selectionBackground", new Color(159, 69, 69));
		UIManager.put("ProgressBar.selectionBackground", new Color(235, 240, 208));
		UIManager.put("ProgressBar.selectionForeground", new Color(159, 0, 0));

		JPanel footer = new JPanel();
		
		plateau = new VuePlateau(modele);
		timer = new JProgressBar();
		score = new JLabel();
		
		Container content = getContentPane();
		setLayout(new BorderLayout());
		
		footer.setLayout(new BorderLayout());
		footer.add(score, BorderLayout.CENTER);
		footer.add(timer, BorderLayout.EAST);
		
		footer.setBackground(new Color(159, 0, 0));
		score.setForeground(Color.WHITE);		

		timer.setForeground(new Color(235, 240, 208));
		timer.setBackground(new Color(212, 97, 97));
		timer.setBorderPainted(false);
		timer.setStringPainted(true);
		timer.setString("");
		
		content.add(plateau, BorderLayout.CENTER);
		content.add(footer, BorderLayout.SOUTH);
		
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		setJMenuBar(new VueMenu(modele));
		
		setTitle("ESIAL English - The Memory edition");
		setSize(1000, 650);
		setVisible(true);
	}
	
	@Override
	public void update(Observable o, Object arg) {
		Update_t u = (Update_t) arg;
		
		if(u == Update_t.TIMER && !modele.partieTerminee()) {
			timer.setValue(modele.getTimeMax() != 0 ? modele.getTimer() * 100 / modele.getTimeMax() : 0);
			
			int remaining = modele.getTimeMax() - modele.getTimer();
			timer.setString(remaining > 0 ? remaining + "s" : "Time out!");
			
			if(modele.tempsEcoule()) {
				JOptionPane.showMessageDialog(this, "Time out!"
						+ "\n\nScore: " + modele.getScore() + " point" + (modele.getScore() > 1 ? "s" : "")
						+ "\nLevel: " + modele.getLevel()
						+ (modele.saveMeilleurScore() ? "\n\nIt's the higher score for this game!" : ""),
						"Game over", JOptionPane.WARNING_MESSAGE, new ImageIcon("icon_time.png"));
				
				plateau.createSplash();
				timer.setValue(0);
				timer.setString("");
				modele.resetScore();
			}
		}
		
		if(u == Update_t.SELECT && modele.partieTerminee()) {
			boolean best = modele.saveMeilleurScore();
			
			JOptionPane.showMessageDialog(this, "Level " + modele.getLevel() + " done."
					+ "\n\nScore: " + modele.getScore() + " point" + (modele.getScore() > 1 ? "s" : "")
					+ "\nBonus: " + modele.getScoreNextLevel() + " point" + (modele.getScoreNextLevel() > 1 ? "s" : "")
					+ (best ? "\n\nIt's the higher score for this game!" : ""),
					"Congratulation", JOptionPane.INFORMATION_MESSAGE, new ImageIcon(best ? "icon_score.png" : "icon_next.png"));
			
			timer.setValue(0);
			timer.setString("");
			
			modele.nextLevel();
		}
		
		if(u == Update_t.SCORE) {
			if(!modele.tempsEcoule()) {
				score.setText("<html><span style='font-size: 12px; color: #ebf0d0'>&nbsp;" +
						"<span style='color: #bbbbbb'>level " + modele.getLevel() + "</span> &nbsp;<b>" + modele.getName() + "</b> (" + modele.getScore() + " point" + (modele.getScore() > 1 ? "s" : "") + ")"
						+ "&nbsp;</span></html>");
			} else
				score.setText("");
			
			repaint();
		}
	}
}