package Anglais;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.util.ArrayList;
import java.util.Observable;
import java.util.Observer;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import Anglais.VueMain.Update_t;

public class VuePlateau extends JPanel implements Observer {
	private static final long serialVersionUID = 2880869631847078944L;
	private Plateau modele;
	private ArrayList<JButton> buttons;
	private ArrayList<JLabel> bLabels;
	
	public VuePlateau(Plateau modele) {
		this.modele = modele;
		createSplash();
		modele.addObserver(this);
		update(modele, null);
	}
	
	public void createSplash() {
		removeAll();
		
		setLayout(new GridBagLayout());
		add(new JLabel(new ImageIcon("splash.png")), new GridBagConstraints());

		revalidate();
		repaint();
	}
	
	private void createButtons() {
		int height, width;
		
		removeAll();

		for(width = (int) Math.sqrt(modele.getSize()); modele.getSize() % width != 0; width--);
		height = modele.getSize() / width;
		
		if(height < width)
			setLayout(new GridLayout(width, height));
		else
			setLayout(new GridLayout(height, width));
		
		buttons = new ArrayList<JButton>();
		bLabels = new ArrayList<JLabel>();
		
		for(Carte c : modele.getCartes()) {
			JButton button = new JButton(new ImageIcon("card.png"));
			JLabel bLabel = new JLabel(c.getFace().endsWith(".png") ? "" : c.getFace());

			button.addMouseListener(new ControleurCarte(modele, c));
			button.setLayout(new GridBagLayout());
			button.add(bLabel, new GridBagConstraints());

			button.setBorderPainted(false);
			button.setFocusable(false);
			button.setOpaque(false);
			button.setContentAreaFilled(false);
			button.setDisabledIcon(new ImageIcon("cardBack.png"));
			
			bLabel.setForeground(new Color(159, 0, 0));
			
			if(c.getFace().endsWith(".png")) {
				bLabel.setPreferredSize(new Dimension(90, 60));
				bLabel.setDisabledIcon(new ImageIcon("data/" + c.getFace()));
			}
			
			buttons.add(button);
			bLabels.add(bLabel);
			add(button);
		}
	}
	
	@Override
	public void update(Observable o, Object arg) {
		Update_t u = (Update_t) arg;
		
		if(u == Update_t.CREATE)
			createButtons();

		if(u == Update_t.CREATE || u == Update_t.SELECT) {
			for(int i = 0; i < buttons.size(); i++) {
				if(!modele.getCarte(i).getFace().endsWith(".png"))
					bLabels.get(i).setText(modele.getCarte(i).getEtat() ? modele.getCarte(i).getFace() : "");
				else
					bLabels.get(i).setEnabled(!modele.getCarte(i).getEtat());

				buttons.get(i).setEnabled(!modele.getCarte(i).getEtat());
			}
		}
	}
}